import { WxTokenResponse, fetchToken, getUserInfo } from '@/modules/wx.ts';
import { useContextKey } from '@kevisual/use-config/context';
import { UserModel } from '@kevisual/code-center-module';
import { Buffer } from 'buffer';
import { CustomError } from '@kevisual/router';
import { customAlphabet } from 'nanoid';
const nanoid = customAlphabet('abcdefghijklmnopqrstuvwxyz', 10);
const User = useContextKey<typeof UserModel>('UserModel');
export class WxServices {
  wxToken: WxTokenResponse;
  // 创建一个webToken，用户登录
  webToken: string;
  accessToken: string;
  refreshToken: string;
  isNew: boolean;
  // @ts-ignore
  user: User;
  constructor() {
    //
  }
  async checkUserExist(username: string) {
    const user = await User.findOne({
      where: {
        username,
      },
    });
    return !!user;
  }
  async randomUsername() {
    const a = customAlphabet('abcdefghijklmnopqrstuvwxyz', 10);
    const b = customAlphabet('1234567890', 10);
    const random = '@' + a(4) + '' + b(4);
    const user = await this.checkUserExist(random);
    if (user) {
      return this.randomUsername();
    }
    return random;
  }
  async login(code: string) {
    const token = await fetchToken(code);
    console.log('login token', token);
    if (!token.unionid) {
      throw new CustomError(400, 'code is invalid, wxdata can not be found');
    }
    this.wxToken = token;
    const unionid = token.unionid;
    let user = await User.findOne({
      where: {
        data: {
          wxUnionId: unionid,
        },
      },
    });
    // @ts-ignore
    if (user && user.data.wxOpenid !== token.openid) {
      user.data = {
        ...user.data,
        // @ts-ignore
        wxOpenid: token.openid,
      };
      await user.update({ data: user.data });
      console.log('mp-user login openid update=============', token.openid, token.unionid);
    }
    if (!user) {
      const username = await this.randomUsername();
      user = await User.createUser(username, nanoid(10));
      user.data = {
        ...user.data,
        // @ts-ignore
        wxOpenid: token.openid,
        wxUnionId: unionid,
      };
      await user.save({ fields: ['data'] });
      this.isNew = true;
    }

    const tokenInfo = await user.createToken(null, 'plugin', {
      wx: {
        openid: token.openid,
        unionid: unionid,
      },
    });
    this.webToken = tokenInfo.accessToken;

    this.accessToken = tokenInfo.accessToken;
    this.refreshToken = tokenInfo.refreshToken;
    this.user = user;
    return {
      accessToken: this.accessToken,
      refreshToken: this.refreshToken,
      isNew: this.isNew,
    };
  }

  async checkHasUser() {}
  async getUserInfo() {
    try {
      if (!this.wxToken) {
        throw new CustomError(400, 'wxToken is not set');
      }
      const userInfo = await getUserInfo(this.wxToken.access_token, this.wxToken.openid);
      const { nickname, headimgurl } = userInfo;
      this.user.nickname = nickname;
      try {
        const downloadImgUrl = await this.downloadImg(headimgurl);
        this.user.avatar = downloadImgUrl;
      } catch (error) {
        console.error('Error downloading or converting image:', error);
      }
      await this.user.save();
    } catch (error) {
      console.error('Error getting user info:', error);
    }
  }
  /**
   * 转成base64
   * @param url
   */
  async downloadImg(url: string): Promise<string> {
    try {
      return await downloadImag(url);
    } catch (error) {
      console.error('Error downloading or converting image:', error);
      throw error;
    }
  }
}

// https://thirdwx.qlogo.cn/mmopen/vi_32/WvOPpbDwUKEVJvSst8Z91Y68m7CsBeecMqRGlqey5HejByePD89boYGaVCM8vESsYmokk1jABUDsK08IrfI6JEkibZkDIC2zsb96DGBTEF7E/132
export const downloadImag = async (url: string) => {
  try {
    const res = await fetch(url);
    if (!res.ok) {
      throw new Error(`HTTP error! Status: ${res.status}`);
    }
    const arrayBufferToBase64 = (buffer: ArrayBuffer): string => {
      const bytes = new Uint8Array(buffer);
      let binary = '';
      for (let i = 0; i < bytes.byteLength; i++) {
        binary += String.fromCharCode(bytes[i]);
      }
      return Buffer.from(binary, 'binary').toString('base64');
    };

    const buffer = await res.arrayBuffer();
    return `data:image/jpeg;base64,${arrayBufferToBase64(buffer)}`;
  } catch (error) {
    console.error('Error downloading or converting image:', error);
    throw error;
  }
};
